@@ -19,6 +19,7 @@
  */
 
 #include <config.h>
+#include <gdk/gdkcolor.h>
 
 #include "evolution-webcal-notify.h"
 
@@ -176,7 +177,7 @@ void e_webcal_display_error (const gchar * title, const gchar * message,
 }
 
 static void select_source (ESource * source, icalcomponent_kind kind) {
-  const char * key = NULL;
+  const gchar * key = NULL;
   GConfClient * config = NULL;
   GSList * list;
 
@@ -208,6 +209,9 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   GtkWidget * tspinb, * toption;
   GtkWidget * menu, * mitem;
   GtkWidget * vbox, * hbox;
+  GtkWidget * lbox, * wbox, * xbox;
+  GtkWidget * rlabel, * clabel;
+  GtkWidget * cbutton;
   GtkWidget * label, * button;
   SoupUri * tmpuri;
   gchar * tmpname;
@@ -217,7 +221,10 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   ESourceGroup * events_group = NULL;
   ESourceGroup * tasks_group = NULL;
   ESourceList * events_sources, * tasks_sources;
+  GdkColor color;
   GSList * l;
+  gchar * scolor;
+  guint32 icolor = 0;
 
   events_sources = e_source_list_new_for_gconf_default (CALENDAR_SOURCES_KEY);
 
@@ -249,10 +256,41 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
     e_source_list_add_group (tasks_sources, tasks_group, -1);
   }
 
-
   tmpuri = soup_uri_new (caluri);
   tmpname = g_path_get_basename (tmpuri->path);
 
+  if (has_events)
+    if (e_source_group_peek_source_by_name (events_group,
+					    name ? name : tmpname)) {
+      has_events = FALSE;
+    }
+
+  if (has_tasks)
+    if (e_source_group_peek_source_by_name (tasks_group,
+					    name ? name : tmpname)) {
+      has_tasks = FALSE;
+    }
+
+  /* If we are already subscribed, pop up an error dialog */
+  if (!has_events && !has_tasks) {
+    gchar * msg;
+
+    msg = g_strdup_printf (_("You are already subscribed to the calendar, "
+			     "\"%s\". Please subscribe to another calendar."),
+			   name ? name : tmpname);
+    e_webcal_display_error (_("<b>Already Subscribed</b>"), msg, NULL);
+    g_free (msg);
+
+    g_free (tmpname);
+    soup_uri_free (tmpuri);
+    
+    g_object_unref (events_sources);
+    g_object_unref (tasks_sources);
+
+    bonobo_main_quit ();
+    return;
+  }
+
   source = e_source_new (name ? name : tmpname, caluri);
 
   dialog = g_new0 (EIcalShareDialog, 1);
@@ -335,8 +373,8 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   gtk_widget_show (dialog->image);
 
   /* Box for Name and Description */
-  vbox = gtk_vbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+  vbox = gtk_vbox_new (FALSE, 12);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
   gtk_widget_show (vbox);
 
   /* Name */
@@ -348,47 +386,52 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
-  /* Box with lame spacers for description */
-  hbox = gtk_hbox_new (FALSE, 12);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-  gtk_widget_show (hbox);
-
-  /* Lame spacer */
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
   /* Description */
   label = gtk_label_new (desc ? desc : _("No Description"));
+  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
-  /* Box with lame spacers for Refresh timeout */
+  /* Box for extra input label and widget boxes */
+  xbox = gtk_hbox_new (FALSE, 12);
+  gtk_box_pack_start (GTK_BOX (vbox), xbox, FALSE, FALSE, 0);
+  gtk_widget_show (xbox);
+
+  /* Box for labels of extra inputs */
+  lbox = gtk_vbox_new (TRUE, 6);
+  gtk_box_pack_start (GTK_BOX (xbox), lbox, FALSE, FALSE, 0);
+  gtk_widget_show (lbox);
+
+  /* Refresh entry label */
+  rlabel = gtk_label_new_with_mnemonic (_("_Refresh Every:"));
+  gtk_misc_set_alignment (GTK_MISC (rlabel), 0, 0.5);
+  gtk_box_pack_start (GTK_BOX (lbox), rlabel, FALSE, FALSE, 0);
+  gtk_widget_show (rlabel);
+
+  /* Color picker label */
+  clabel = gtk_label_new_with_mnemonic (_("C_olor:"));
+  gtk_misc_set_alignment (GTK_MISC (clabel), 0, 0.5);
+  gtk_box_pack_start (GTK_BOX (lbox), clabel, FALSE, FALSE, 0);
+  gtk_widget_show (clabel);
+
+  /* Box for widgets of extra inputs */
+  wbox = gtk_vbox_new (TRUE, 6);
+  gtk_box_pack_start (GTK_BOX (xbox), wbox, FALSE, FALSE, 0);
+  gtk_widget_show (wbox);
+
+  /* Box for Refresh timeout */
   hbox = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (wbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  /* Lame spacer */
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  label = gtk_label_new ("");
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
   /* We need to ask the user when to Refresh the calendar */
-  label = gtk_label_new_with_mnemonic (_("_Refresh Every:"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
   spinadj = gtk_adjustment_new (12, 1, 23, 1, 1, 1);
   tspinb = gtk_spin_button_new (GTK_ADJUSTMENT (spinadj), 12, 0);
   gtk_box_pack_start (GTK_BOX (hbox), tspinb, FALSE, FALSE, 0);
   gtk_widget_show (tspinb);
 
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), tspinb);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (rlabel), tspinb);
 
   toption = gtk_option_menu_new ();
   gtk_box_pack_start (GTK_BOX (hbox), toption, FALSE, FALSE, 0);
@@ -421,6 +464,21 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   g_signal_connect (G_OBJECT (menu), "deactivate",
 		    G_CALLBACK (e_webcal_change_adjustment), tspinb);
 
+  /* Color button for the calendar properties */
+  hbox = gtk_hbox_new (FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (wbox), hbox, FALSE, FALSE, 0);
+  gtk_widget_show (hbox);
+
+  cbutton = gtk_color_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox), cbutton, FALSE, FALSE, 0);
+  gtk_widget_show (cbutton);
+
+  scolor = g_strdup_printf ("#%s", e_source_get_property (source, "color"));
+  gdk_color_parse (scolor, &color);
+  gtk_color_button_set_color (GTK_COLOR_BUTTON (cbutton), &color);
+  g_free (scolor);
+
+  gtk_label_set_mnemonic_widget (GTK_LABEL (clabel), cbutton);
 
   g_free (icofile);
   g_free (tmpname);
@@ -449,6 +507,10 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
     e_source_set_property (source, "refresh", ref_str);
     g_free (ref_str);
 
+    gtk_color_button_get_color (GTK_COLOR_BUTTON (cbutton), &color);
+    icolor = ((color.red >> 8) << 16) | ((color.green >> 8) << 8) | (color.blue >> 8);
+    e_source_set_color (source, icolor);
+
     if (!strncmp (caluri, WEBCAL_BASE_URI, 9)) {
       e_source_set_relative_uri (source, caluri + 9);
     } else if (!strncmp (caluri, "http://", 7)) {
